@@ -1,5 +1,5 @@
 /* User defined objects */
-/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002, 2003 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -784,7 +784,7 @@ int SLclass_set_pop_function (SLang_Class_Type *cl, int (*f)(unsigned char, VOID
    return 0;
 }
 
-int SLclass_set_push_function (SLang_Class_Type *cl, int (*f)(unsigned char, VOID_STAR))
+int SLclass_set_push_function (SLang_Class_Type *cl, int (*f)(SLtype, VOID_STAR))
 {
    if (cl == NULL) return -1;
    cl->cl_push = f;
@@ -792,6 +792,14 @@ int SLclass_set_push_function (SLang_Class_Type *cl, int (*f)(unsigned char, VOI
    return 0;
 }
 
+int SLclass_set_apush_function (SLang_Class_Type *cl, int (*f)(SLtype, VOID_STAR))
+{
+   if (cl == NULL) return -1;
+   cl->cl_apush = f;
+
+   return 0;
+}
+
 int SLclass_set_acopy_function (SLang_Class_Type *cl, int (*f)(SLtype, VOID_STAR, VOID_STAR))
 {
    if (cl == NULL) return -1;
@@ -1059,7 +1067,16 @@ SLclass_typecast (unsigned char to_type, int is_implicit, int allow_array)
 
    if (1 == status)
      {
-	if (-1 == (*cl_to->cl_apush)(to_type, bp))
+	/* AnyType apush will do a reference, which is undesirable here.
+	 * So, to avoid that, perform push instead of apush.  Yes, this is
+	 * an ugly hack.
+	 */
+	if (to_type == SLANG_ANY_TYPE)
+	  status = (*cl_to->cl_push)(to_type, bp);
+	else 
+	  status = (*cl_to->cl_apush)(to_type, bp);
+	
+	if (status == -1)
 	  {
 	     (*cl_to->cl_adestroy) (to_type, bp);
 	     SLang_free_object (&obj);
