@@ -1,4 +1,4 @@
-/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002, 2003 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -161,6 +161,16 @@ static Ansi_Color_Type Ansi_Color_Map[JMAX_COLORS] =
      {RGB(0, 1, 0, 1, 1, 1), SLTT_REV_MASK, NULL}
 };
 
+/* 0 if least significant bit is blue, not red */
+static int Is_Fg_BGR = 0;
+static int Is_Bg_BGR = 0;
+#define COLOR_ARG(color, is_bgr) ((is_bgr) ? RGB_to_BGR[(color)&0x7] : (color))
+static SLCONST int RGB_to_BGR[] =
+{
+     0, 4, 2, 6, 1, 5, 3, 7
+};
+
+
 static char *Color_Fg_Str = "\033[3%dm";
 static char *Color_Bg_Str = "\033[4%dm";
 static char *Default_Color_Fg_Str = "\033[39m";
@@ -778,6 +788,9 @@ void SLtt_erase_line (void)
    tt_write_string("\r");
    Cursor_Set = 1; Cursor_c = 0;
    SLtt_del_eol();
+   /* Put the cursor back at the beginning of the line */
+   tt_write_string("\r");
+   Cursor_Set = 1; Cursor_c = 0;
 }
 
 /* It appears that the Linux console, and most likely others do not
@@ -928,7 +941,7 @@ void SLtt_del_eol (void)
    del_eol ();
 }
 
-typedef struct
+typedef SLCONST struct
 {
    char *name;
    SLtt_Char_Type color;
@@ -971,10 +984,10 @@ void SLtt_set_mono (int obj, char *what, SLtt_Char_Type mask)
 static char *check_color_for_digit_form (char *color)
 {
    unsigned int i, ich;
-   char *s = color;
+   unsigned char *s = (unsigned char *) color;
 
    i = 0;
-   while ((ich = (int) *s) != 0)
+   while ((ich = (unsigned int) *s) != 0)
      {
 	if ((ich < '0') || (ich > '9'))
 	  return color;
@@ -1031,7 +1044,7 @@ static int get_default_colors (char **fgp, char **bgp)
    p = bg_buf;
    pmax = p + (sizeof (bg_buf) - 1);
 
-   /* Mark suggested allowing for extra spplication specific stuff following
+   /* Mark suggested allowing for extra application specific stuff following
     * the background color.  That is what the check for the semi-colon is for.
     */
    while ((*bg != 0) && (*bg != ';'))
@@ -1129,7 +1142,7 @@ static SLtt_Char_Type fb_to_fgbg (SLtt_Char_Type f, SLtt_Char_Type b)
 }
 
 /* This looks for colors with name form 'colorN'.  If color is of this
- * form, N is passed back via paramter list.
+ * form, N is passed back via parameter list.
  */
 static int parse_color_digit_name (char *color, SLtt_Char_Type *f)
 {
@@ -1146,12 +1159,20 @@ static int parse_color_digit_name (char *color, SLtt_Char_Type *f)
    i = 0;
    while (1)
      {
+	unsigned int j;
+
 	ch = (unsigned char) *color++;
 	if (ch == 0)
 	  break;
 	if ((ch > '9') || (ch < '0'))
 	  return -1;
-	i = 10 * i + (ch - '0');
+	
+	if (i > 0xFFFFFFFFU / 10)
+	  return -1;
+	j = (i *= 10);
+	i += (ch - '0');
+	if (i < j)
+	  return -1;
      }
 
    *f = (SLtt_Char_Type) i;
@@ -1332,7 +1353,7 @@ static void write_attributes (SLtt_Char_Type fgbg)
 	     if (fg0 == SLSMG_COLOR_DEFAULT)
 	       tt_write_string (Default_Color_Fg_Str);
 	     else
-	       tt_printf (Color_Fg_Str, fg0, 0);
+	       tt_printf (Color_Fg_Str, COLOR_ARG(fg0, Is_Bg_BGR), 0);
 	  }
 
 	if (unknown_attributes
@@ -1341,7 +1362,7 @@ static void write_attributes (SLtt_Char_Type fgbg)
 	     if (bg0 == SLSMG_COLOR_DEFAULT)
 	       tt_write_string (Default_Color_Bg_Str);
 	     else
-	       tt_printf (Color_Bg_Str, bg0, 0);
+	       tt_printf (Color_Bg_Str, COLOR_ARG(bg0, Is_Bg_BGR), 0);
 	  }
      }
 
@@ -1469,7 +1490,7 @@ static void write_string_with_care (char *str)
 
 static void send_attr_str (SLsmg_Char_Type *s)
 {
-   unsigned char out[256], ch, *p;
+   unsigned char out[SLTT_MAX_SCREEN_COLS], ch, *p;
    register SLtt_Char_Type attr;
    register SLsmg_Char_Type sh;
    int color, last_color = -1;
@@ -1589,7 +1610,7 @@ static void forward_cursor (unsigned int n, int row)
 void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int row)
 {
    register SLsmg_Char_Type *p, *q, *qmax, *pmax, *buf;
-   SLsmg_Char_Type buffer[256];
+   SLsmg_Char_Type buffer[SLTT_MAX_SCREEN_COLS+1];
    unsigned int n_spaces;
    SLsmg_Char_Type *space_match, *last_buffered_match;
 #ifdef HP_GLITCH_CODE
@@ -1615,6 +1636,9 @@ void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int
      }
 #endif
      
+   if (len > SLTT_MAX_SCREEN_COLS)
+     len = SLTT_MAX_SCREEN_COLS;
+
    q = oldd; p = neww;
    qmax = oldd + len;
    pmax = p + len;
@@ -2158,6 +2182,10 @@ int SLtt_initialize (char *term)
 	  return -1;
      }
 
+   if (_SLsecure_issetugid ()
+       && ((term[0] == '.') || (NULL != strchr(term, '/'))))
+     return -1;
+   
    Linux_Console = (!strncmp (term, "linux", 5)
 # ifdef linux
 		    || !strncmp(term, "con", 3)
@@ -2361,12 +2389,21 @@ int SLtt_initialize (char *term)
      SLtt_Has_Alt_Charset = 0;
 
    Reset_Color_String = SLtt_tgetstr ("op");
+   
+   /* Apparantly the difference between "AF" and "Sf" is that AF uses RGB, 
+    * but Sf uses BGR.
+    */
    Color_Fg_Str = SLtt_tgetstr ("AF"); /* ANSI setaf */
-   Color_Bg_Str = SLtt_tgetstr ("AB"); /* ANSI setbf */
-   if ((Color_Fg_Str == NULL) || (Color_Bg_Str == NULL))
+   if (Color_Fg_Str == NULL)
      {
 	Color_Fg_Str = SLtt_tgetstr ("Sf");   /* setf */
+	Is_Fg_BGR = (Color_Fg_Str != NULL);
+     }
+   Color_Bg_Str = SLtt_tgetstr ("AB"); /* ANSI setbf */
+   if (Color_Bg_Str == NULL)
+     {
 	Color_Bg_Str = SLtt_tgetstr ("Sb");   /* setb */
+	Is_Fg_BGR = (Color_Bg_Str != NULL);
      }
 
    if ((Max_Terminfo_Colors = SLtt_tgetnum ("Co")) < 0)
@@ -2636,12 +2673,8 @@ void SLtt_get_screen_size (void)
 	if (s != NULL) c = atoi (s);
      }
 
-   if (r <= 0) r = 24;
-   if (c <= 0) c = 80;
-#if 0
-   if ((r <= 0) || (r > 200)) r = 24;
-   if ((c <= 0) || (c > 250)) c = 80;
-#endif
+   if ((r <= 0) || (r > SLTT_MAX_SCREEN_ROWS)) r = 24;
+   if ((c <= 0) || (c > SLTT_MAX_SCREEN_COLS)) c = 80;
    SLtt_Screen_Rows = r;
    SLtt_Screen_Cols = c;
 }
