@@ -1,4 +1,4 @@
-/* Copyright (c) 1998, 1999, 2001, 2002 John E. Davis
+/* Copyright (c) 1998, 1999, 2001, 2002, 2003 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -160,17 +160,20 @@ static int import_from_library (char *name,
 
    fun_name = ns_init_fun_name;
    ns_init_fun = (int (*)(char *)) dlsym (handle, fun_name);
+
+   if ((ns == NULL) || (*ns == 0))
+     ns = "Global";
+
    if (ns_init_fun == NULL) 
      {
-	if ((ns != NULL) 
-	    && (0 != strcmp (ns, "Global")))
+	if (0 != strcmp (ns, "Global"))
 	  goto return_error;
-	
+
 	fun_name = init_fun_name;
 	init_fun = (int (*)(void)) dlsym (handle, fun_name);
 	if (init_fun == NULL)
 	  goto return_error;
-	
+
 	if (-1 == (*init_fun) ())
 	  {
 	     dlclose (handle);
