@@ -1,5 +1,5 @@
 /* Pathname and filename functions */
-/* Copyright (c) 1998, 1999, 2001, 2002 John E. Davis
+/* Copyright (c) 1998, 1999, 2001, 2002, 2003 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -59,6 +59,8 @@
 # define IS_PATH_SEP(x)	(((x) == PATH_SEP) || ((x) == '/'))
 #endif
 
+static char Path_Delimiter = SEARCH_PATH_DELIMITER;
+
 /* If file is /a/b/c/basename, this function returns a pointer to basename */
 char *SLpath_basename (char *file)
 {
@@ -179,6 +181,9 @@ static void convert_slashes (char *f)
 
 int SLpath_is_absolute_path (char *name)
 {
+   if (name == NULL)
+     return -1;
+
 #ifdef UNIX_PATHNAMES_OK
    return (*name == '/');
 #else
@@ -200,6 +205,7 @@ int SLpath_is_absolute_path (char *name)
 #endif
 }
 
+
 /* This returns a MALLOCED string */
 char *SLpath_dircat (char *dir, char *name)
 {
@@ -285,6 +291,25 @@ int SLpath_file_exists (char *file)
    return 1;
 }
 
+/* By relatively-absolute, I mean paths of the form ./foo,
+ * and ../foo/bar.  But not foo/bar.
+ */
+static int is_relatively_absolute (char *file)
+{
+   if (file == NULL)
+     return -1;
+   if (SLpath_is_absolute_path (file))
+     return 1;
+   
+#if defined(VMS)
+   return (*file == '[');
+#else
+   if (*file == '.') file++;
+   if (*file == '.') file++;
+   return (*file == PATH_SEP);
+#endif
+}
+
 char *SLpath_find_file_in_path (char *path, char *name)
 {
    unsigned int max_path_len;
@@ -296,13 +321,28 @@ char *SLpath_find_file_in_path (char *path, char *name)
    if ((path == NULL) || (*path == 0)
        || (name == NULL) || (*name == 0))
      return NULL;
+   
+   if (is_relatively_absolute (name))
+     {
+	if (0 == SLpath_file_exists (name))
+	  return NULL;
+	return SLmake_string (name);
+     }
 
+   /* Allow "." to mean the current directory on all systems */
+   if ((path[0] == '.') && (path[1] == 0))
+     {
+	if (0 == SLpath_file_exists (name))
+	  return NULL;
+	return SLpath_dircat (THIS_DIR_STRING, name);
+     }
+   
    max_path_len = 0;
    this_path_len = 0;
    p = path;
    while (*p != 0)
      {
-	if (*p++ == SEARCH_PATH_DELIMITER)
+	if (*p++ == Path_Delimiter)
 	  {
 	     if (this_path_len > max_path_len) max_path_len = this_path_len;
 	     this_path_len = 0;
@@ -316,7 +356,7 @@ char *SLpath_find_file_in_path (char *path, char *name)
      return NULL;
 
    nth = 0;
-   while (-1 != SLextract_list_element (path, nth, SEARCH_PATH_DELIMITER,
+   while (-1 != SLextract_list_element (path, nth, Path_Delimiter,
 					dir, max_path_len))
      {
 	nth++;
@@ -342,3 +382,17 @@ char *SLpath_find_file_in_path (char *path, char *name)
    return NULL;
 }
 
+int SLpath_get_delimiter (void)
+{
+   return Path_Delimiter;
+}
+
+int SLpath_set_delimiter (int d)
+{
+   char ch = (char) d;
+   if (ch == 0)
+     return -1;
+   
+   Path_Delimiter = ch;
+   return 0;
+}
