@@ -1,5 +1,5 @@
 /* SLang Screen management routines */
-/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002, 2003 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -26,13 +26,7 @@ Screen_Type;
 #define TRASHED 0x2
 static int Screen_Trashed;
 
-#if !defined(__MSDOS_16BIT__)
-# define MAX_SCREEN_SIZE 256
-#else
-# define MAX_SCREEN_SIZE 75
-#endif
-
-Screen_Type SL_Screen[MAX_SCREEN_SIZE];
+Screen_Type SL_Screen[SLTT_MAX_SCREEN_ROWS];
 static int Start_Col, Start_Row;
 static int Screen_Cols, Screen_Rows;
 static int This_Row, This_Col;
@@ -959,12 +953,13 @@ void SLsmg_touch_screen (void)
 
 			  
 #ifndef IBMPC_SYSTEM
-static char Fake_Alt_Char_Pairs [] = "a:j+k+l+m+q-t+u+v+w+x|n+`+f\\g#~o,<+>.v-^h#0#";
+static SLCONST char Fake_Alt_Char_Pairs [] = "a:j+k+l+m+q-t+u+v+w+x|n+`+f\\g#~o,<+>.v-^h#0#";
 
 static void init_alt_char_set (void)
 {
    int i;
-   unsigned char *p, *pmax, ch;
+   unsigned SLCONST char *p, *pmax;
+   unsigned char ch;
 
    if (Alt_Char_Set[128] == 128) return;
 
@@ -983,8 +978,8 @@ static void init_alt_char_set (void)
 	else p = (unsigned char *) *tt_Graphics_Char_Pairs;
 	if (p == NULL) return;
      }
-   else	p = (unsigned char *) Fake_Alt_Char_Pairs;
-   pmax = p + strlen ((char *) p);
+   else	p = (unsigned SLCONST char *) Fake_Alt_Char_Pairs;
+   pmax = p + strlen ((SLCONST char *) p);
 
    /* Some systems have messed up entries for this */
    while (p < pmax)
@@ -1076,8 +1071,8 @@ static int init_smg (void)
 #endif
 
    Screen_Rows = *tt_Screen_Rows;
-   if (Screen_Rows > MAX_SCREEN_SIZE)
-     Screen_Rows = MAX_SCREEN_SIZE;
+   if (Screen_Rows > SLTT_MAX_SCREEN_ROWS)
+     Screen_Rows = SLTT_MAX_SCREEN_ROWS;
 
    Screen_Cols = *tt_Screen_Cols;
 
