@@ -1,5 +1,5 @@
 /* Structure type implementation */
-/* Copyright (c) 1998, 1999, 2001, 2002 John E. Davis
+/* Copyright (c) 1998, 1999, 2001, 2002, 2003 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -968,13 +968,13 @@ static _SLstruct_Field_Type *find_field_via_strcmp (_SLang_Struct_Type *s, char
    return NULL;
 }
 
-static void free_cstruct_field (SLang_CStruct_Field_Type *cfield, char *cs)
+static void free_cstruct_field (SLang_CStruct_Field_Type *cfield, VOID_STAR cs)
 {
    SLang_Class_Type *cl;
 
    if ((cfield->read_only == 0) 
        && (NULL != (cl = _SLclass_get_class (cfield->type))))
-     _SLarray_free_array_elements (cl, (VOID_STAR)(cs + cfield->offset), 1);
+     _SLarray_free_array_elements (cl, (VOID_STAR)((char*)cs + cfield->offset), 1);
 }
 
 void SLang_free_cstruct (VOID_STAR cs, SLang_CStruct_Field_Type *cfields)
@@ -984,7 +984,7 @@ void SLang_free_cstruct (VOID_STAR cs, SLang_CStruct_Field_Type *cfields)
 
    while (cfields->field_name != NULL)
      {
-	free_cstruct_field (cfields, (char *) cs);
+	free_cstruct_field (cfields, cs);
 	cfields++;
      }
 }
