@@ -2,7 +2,7 @@
  * the slang SLtt interface.
  */
 
-/* Copyright (c) 1992, 1999, 2001, 2002 John E. Davis
+/* Copyright (c) 1992, 1999, 2001, 2002, 2003 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -249,6 +249,10 @@ SLterminfo_Type *_SLtt_tigetent (char *term)
        )
      return NULL;
 
+   if (_SLsecure_issetugid ()
+       && ((term[0] == '.') || (NULL != strchr (term, '/'))))
+     return NULL;
+
    if (NULL == (ti = (SLterminfo_Type *) SLmalloc (sizeof (SLterminfo_Type))))
      {
 	return NULL;
@@ -265,7 +269,7 @@ SLterminfo_Type *_SLtt_tigetent (char *term)
    /* If we are on a termcap based system, use termcap */
    if (0 == tcap_getent (term, ti)) return ti;
 
-   if (NULL != (home = getenv ("HOME")))
+   if (NULL != (home = _SLsecure_getenv ("HOME")))
      {
 	strncpy (home_ti, home, sizeof (home_ti) - 11);
 	home_ti [sizeof(home_ti) - 11] = 0;
@@ -273,7 +277,7 @@ SLterminfo_Type *_SLtt_tigetent (char *term)
 	Terminfo_Dirs [0] = home_ti;
      }
 
-   Terminfo_Dirs[1] = getenv ("TERMINFO");
+   Terminfo_Dirs[1] = _SLsecure_getenv ("TERMINFO");
    i = 0;
    while (1)
      {
@@ -335,7 +339,7 @@ SLterminfo_Type *_SLtt_tigetent (char *term)
 # define UNTIC_COMMENT(x)
 #endif
 
-typedef struct
+typedef SLCONST struct
 {
    char name[3];
    int offset;
@@ -1003,6 +1007,16 @@ static int tcap_getent (char *term, SLterminfo_Type *ti)
    termcap = (unsigned char *) getenv ("TERMCAP");
    if ((termcap == NULL) || (*termcap == '/')) return -1;
 
+   /* SUN Solaris 7&8 have bug in tset program under tcsh,
+    * eval `tset -s -A -Q` sets value of TERMCAP to ":",
+    *  under other shells it works fine.
+    *  SUN was informed, they marked it as duplicate of bug 4086585
+    *  but didn't care to fix it... <mikkopa@cs.tut.fi> 
+    */
+   if ((termcap[0] == ':') && (termcap[1] == 0))
+     return -1;
+   
+   
    /* We have a termcap so lets use it provided it does not have a reference
     * to another terminal via tc=.  In that case, use terminfo.  The alternative
     * would be to parse the termcap file which I do not want to do right now.
