@@ -1,4 +1,4 @@
-/* Copyright (c) 1998, 1999, 2001, 2002 John E. Davis
+/* Copyright (c) 1998, 1999, 2001, 2002, 2003 John E. Davis
  * This file is part of the S-Lang library.
  *
  * You may distribute under the terms of either the GNU General Public
@@ -1066,6 +1066,14 @@ static char *read_from_file (SLang_Load_Type *x)
    return fgets (c->buf, MAX_FILE_LINE_LEN, c->fp);
 }
 
+static int Load_File_Verbose = 0;
+int SLang_load_file_verbose (int v)
+{
+   int v1 = Load_File_Verbose;
+   Load_File_Verbose = v;
+   return v1;
+}
+
 /* Note that file could be freed from Slang during run of this routine
  * so get it and store it !! (e.g., autoloading)
  */
@@ -1089,9 +1097,11 @@ int SLns_load_file (char *f, char *ns_name)
    if (SLns_Load_File_Hook != NULL)
      return (*SLns_Load_File_Hook) (f, ns_name);
 
-   if (f == NULL) name = "<stdin>"; else name = f;
-
-   name = SLang_create_slstring (name);
+   if (f == NULL) 
+     name = SLang_create_slstring ("<stdin>");
+   else
+     name = _SLpath_find_file (f);
+	
    if (name == NULL)
      return -1;
 
@@ -1104,7 +1114,11 @@ int SLns_load_file (char *f, char *ns_name)
    buf = NULL;
 
    if (f != NULL)
-     fp = fopen (f, "r");
+     {
+	fp = fopen (name, "r");
+	if (Load_File_Verbose)
+	  SLang_vmessage ("Loading %s", name);
+     }
    else
      fp = stdin;
 
